import esp32
from micropython import const

_INT32_MIN = const(0x80000000)


class TouchCalibration(object):

    def __init__(self, name):
        self._config = esp32.NVS(name)

        try:
            self._left = self._config.get_i32('left')
            if self._left == _INT32_MIN:
                self._left = None

        except OSError:
            self._left = None

        try:
            self._right = self._config.get_i32('right')
            if self._right == _INT32_MIN:
                self._right = None
        except OSError:
            self._right = None
        try:
            self._top = self._config.get_i32('top')
            if self._top == _INT32_MIN:
                self._top = None
        except OSError:
            self._top = None
        try:
            self._bottom = self._config.get_i32('bottom')
            if self._bottom == _INT32_MIN:
                self._bottom = None
        except OSError:
            self._bottom = None

        self._is_dirty = False

    def save(self):
        if self._is_dirty:
            self._config.commit()

    @property
    def left(self):
        return self._left

    @left.setter
    def left(self, value):
        self._left = value

        if value is None:
            value = _INT32_MIN

        self._config.set_i32('left', value)
        self._is_dirty = True

    @property
    def right(self):
        return self._right

    @right.setter
    def right(self, value):
        self._right = value

        if value is None:
            value = _INT32_MIN

        self._config.set_i32('right', value)
        self._is_dirty = True

    @property
    def top(self):
        return self._top

    @top.setter
    def top(self, value):
        self._top = value

        if value is None:
            value = _INT32_MIN

        self._config.set_i32('top', value)
        self._is_dirty = True

    @property
    def bottom(self):
        return self._bottom

    @bottom.setter
    def bottom(self, value):
        self._bottom = value

        if value is None:
            value = _INT32_MIN

        self._config.set_i32('bottom', value)
        self._is_dirty = True

    def reset(self):
        self.top = None
        self.bottom = None
        self.left = None
        self.right = None
        self.save()
