
try:
    from machine import I2C as _I2C, Pin
except ImportError:
    from simulator.machine import I2C as _I2C, Pin


class I2CBus(object):

    def __init__(self, scl, sda):
        if (scl, sda) == (19, 18):
            self._bus = _I2C(0, scl=Pin(scl), sda=Pin(sda))
        else:
            self._bus = _I2C(1, scl=Pin(scl), sda=Pin(sda))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def scan(self):
        return self._bus.scan()

    def start(self):
        self._bus.start()

    def stop(self):
        self._bus.stop()

    def readinto(self, buf, nack=True):
        self._bus.readinto(buf, nack)

    def write(self, buf):
        self._bus.write(buf)

    def readfrom(self, addr, nbytes, stop=True):
        return self._bus.readfrom(addr, nbytes, stop)

    def readfrom_into(self, addr, buf, stop=True):
        self._bus.readfrom_into(addr, buf, stop)

    def writeto(self, addr, buf, stop=True):
        self._bus.writeto(addr, buf, stop)

    def writevto(self, addr, vector, stop=True):
        self._bus.writevto(addr, vector, stop)

    def readfrom_mem(self, addr, memaddr, nbytes, addrsize=8):
        return self._bus.readfrom_mem(addr, memaddr, nbytes, addrsize=addrsize)

    def readfrom_mem_into(self, addr, memaddr, buf, addrsize=8):
        self._bus.readfrom_mem_into(addr, memaddr, buf, addrsize=addrsize)

    def writeto_mem(self, addr, memaddr, buf, *, addrsize=8):
        self._bus.writeto_mem(addr, memaddr, buf, addrsize=addrsize)


class I2CDevice(object):

    def __init__(self, bus, dev_id):
        self._bus = bus
        self.dev_id = dev_id

    def read(self, memaddr, num_bytes=None, buf=None):
        with self._bus:
            if buf is not None:
                self._bus.readfrom_mem_into(
                    self.dev_id,
                    memaddr,
                    buf
                )
                return

            if num_bytes == 1:
                return self._bus.readfrom_mem(
                    self.dev_id, memaddr, num_bytes
                )[0]

            return self._bus.readfrom_mem(self.dev_id, memaddr, num_bytes)

    def write(self, memaddr, buf):
        if isinstance(buf, int):
            buf = bytearray([buf])
        with self._bus:
            self._bus.writeto_mem(self.dev_id, memaddr, buf)

